from prompt_toolkit.validation import Validator, ValidationError
import re


class TypeValidator(Validator):
    def __init__(self, valid_types: list):
        self.valid_types = valid_types
        super().__init__()

    def validate(self, document):
        text = document.text
        if text not in self.valid_types:
            raise ValidationError(message="That is not a valid type.")


class YesNoValidator(Validator):
    def __init__(self, answer_required: bool):
        self.answer_required = answer_required
        self.confirmations = ["y", "yes"]
        self.rejections = ["n", "no"]
        self.valid_responses = self.confirmations + self.rejections
        if not self.answer_required:
            self.valid_responses.append("")

        super().__init__()

    def validate(self, document):
        text = document.text
        if text.lower().strip() not in self.valid_responses:
            raise ValidationError(message="Answer must be yes/no.")


class DescriptionValidator(Validator):
    def __init__(self, maximum_chars: int):
        if maximum_chars < 0:
            raise ValueError("Cannot have sub-zero maximum")
        self.max_chars_allowed = maximum_chars
        super().__init__()

    def validate(self, document):
        text = document.text

        if text.strip() == "":
            raise ValidationError(message="You must write a description.")

        if len(text) > self.max_chars_allowed:
            num_chars_overflowed = len(text) - self.max_chars_allowed
            raise ValidationError(
                message=f"You are {num_chars_overflowed} characters over the limit."
            )


class BodyValidator(Validator):
    def __init__(self, session, is_breaking_change: bool):
        self.session = session
        self.is_breaking_change = is_breaking_change
        super().__init__()

    def validate(self, document):
        text = document.text

        if text.strip() == "":
            self.session.multiline = False
            if self.is_breaking_change:
                raise ValidationError(
                    message="You must write a body for a breaking change."
                )
        else:
            self.session.multiline = True


class FooterValidator(Validator):
    def __init__(self, session):
        self.session = session

    def validate(self, document):
        text = document.text
        if text.strip() == "":
            self.session.multiline = False
        else:
            self.session.multiline = True

            input_lines = text.split("\n")
            for line in input_lines:
                matches_incomplete = [
                    m[0]
                    for m in re.findall(
                        r"(\[\s*(ch|branch(\s*ch)?)\s*[0-9]+\s*\]?)", line
                    )
                ]  # rough matches
                matches_complete = [
                    m[0] for m in re.findall(r"(\[(ch|branch ch)[0-9]+\])", line)
                ]  # strict matches

                if len(matches_incomplete) != len(matches_complete):
                    for match in matches_incomplete:
                        if match not in matches_complete:
                            raise ValidationError(
                                message="An invalid Clubhouse reference was found: "
                                + str(match)
                            )



import requests
import re
import os
from prompt_toolkit import print_formatted_text
from prompt_toolkit.formatted_text import FormattedText
from prompt_toolkit.styles import Style
import pyperclip  # pylint: disable=import-error
from packaging import version
from gitcommit.style import style, Ansi


def get_github_tags():
    resp = requests.get("https://api.github.com/repos/nebbles/gitcommit/tags")
    if resp.status_code == requests.codes["ok"]:
        tags_json = resp.json()
        tags = [tag["name"] for tag in tags_json]
        return tags
    else:
        print("Error fetching tags from GitHub")
        resp.raise_for_status()


def find_version():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    version_file_path = os.path.join(dir_path, "__version__.py")
    with open(version_file_path, "r") as f:
        version_file = f.read()
        version_match = re.search(
            r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
        )
        if version_match:
            version = version_match.group(1)
            return version
        raise RuntimeError("Unable to find version string in __version__.py.")


def check_for_update():
    try:
        tags = get_github_tags()
    except Exception as e:
        print(
            "An error occured whilst checking for updates. Check your network connection."
        )
        return
    latest_tag_version = tags[0][1:]
    cur_version = find_version()

    if version.parse(cur_version) < version.parse(latest_tag_version):
        Ansi.print_ok("There is an update available for conventional-commit.")
        upgrade_command = "pip install --upgrade conventional-commit"
        pyperclip.copy(upgrade_command)

        text = FormattedText(
            [
                ("", "Version "),
                ("class:red", cur_version),
                ("", " → "),
                ("class:green", latest_tag_version),
                ("", "\nAdded to your clipboard: "),
                ("class:command", upgrade_command),
            ]
        )
        print_formatted_text(text, style=style)



from prompt_toolkit.styles import Style

style = Style.from_dict(
    {"green": "#a0d762 bold", "red": "#e67061 bold", "command": "#f78ae0 bold"}
)


class Ansi:
    """
    This class contains the ANSI escape codes for printing to the console as
    well as commonly used combinations of these codes.
    """

    reset = "\033[0m"
    bold = "\033[1m"
    faint = "\033[2m"
    italic = "\033[3m"
    underline = "\033[4m"
    blink = "\033[5m"
    inverse = "\033[7m"

    class fg:
        black = "\033[30m"
        red = "\033[31m"
        green = "\033[32m"
        yellow = "\033[33m"
        blue = "\033[34m"
        magenta = "\033[35m"
        cyan = "\033[36m"
        white = "\033[37m"

        grey = "\033[90m"
        bright_red = "\033[91m"
        bright_green = "\033[92m"
        bright_yellow = "\033[93m"
        bright_blue = "\033[94m"
        bright_magenta = "\033[95m"
        bright_cyan = "\033[96m"
        bright_white = "\033[97m"

    class bg:
        black = "\033[40m"
        red = "\033[41m"
        green = "\033[42m"
        yellow = "\033[43m"
        blue = "\033[44m"
        magenta = "\033[45m"
        cyan = "\033[46m"
        white = "\033[47m"

        grey = "\033[100m"
        bright_red = "\033[101m"
        bright_green = "\033[102m"
        bright_yellow = "\033[103m"
        bright_blue = "\033[104m"
        bright_magenta = "\033[105m"
        bright_cyan = "\033[106m"
        bright_white = "\033[107m"

    @classmethod
    def colour(cls, *args):
        """Combines all args into string. Should pass colour modifiers in first followed by the string. Does not need reset sequence at the end."""
        return "".join(args) + cls.reset

    @classmethod
    def b_green(cls, content):
        return cls.fg.bright_green + cls.bold + str(content) + cls.reset

    @classmethod
    def b_red(cls, content):
        return cls.fg.bright_red + cls.bold + str(content) + cls.reset

    @classmethod
    def b_yellow(cls, content):
        return cls.fg.bright_yellow + cls.bold + str(content) + cls.reset

    @classmethod
    def b_blue(cls, content):
        return cls.fg.bright_blue + cls.bold + str(content) + cls.reset

    @classmethod
    def print_ok(cls, content, end="\n"):
        print(cls.b_green(content), end=end)

    @classmethod
    def print_error(cls, content, end="\n"):
        print(cls.b_red(content), end=end)

    @classmethod
    def print_warning(cls, content, end="\n"):
        print(cls.b_yellow(content), end=end)

    @classmethod
    def print_info(cls, content, end="\n"):
        print(cls.b_blue(content), end=end)



from prompt_toolkit.completion import (
    Completer,
    Completion,
    FuzzyCompleter,
    WordCompleter,
)


class FuzzyWordCompleter(Completer):
    """
    Fuzzy completion on a list of words.
    (This is basically a `WordCompleter` wrapped in a `FuzzyCompleter`.)
    :param words: List of words or callable that returns a list of words.
    :param meta_dict: Optional dict mapping words to their meta-information.
    :param WORD: When True, use WORD characters.
    """

    def __init__(self, words, meta_dict=None, WORD=False):
        # assert callable(words) or all(isinstance(w, string_types) for w in words)

        self.words = words
        self.meta_dict = meta_dict or {}
        self.WORD = WORD

        self.word_completer = WordCompleter(
            words=lambda: self.words, WORD=self.WORD, meta_dict=self.meta_dict
        )

        self.fuzzy_completer = FuzzyCompleter(self.word_completer, WORD=self.WORD)

    def get_completions(self, document, complete_event):
        return self.fuzzy_completer.get_completions(document, complete_event)


class TypeCompleter(FuzzyWordCompleter):
    def __init__(self):
        self.meta_dict = {
            "feat": "production code feature",
            "fix": "production code fixing",
            "chore": "build system / dependencies / config / scripts",
            "docs": "documentation / comments",
            "perf": "improves performance",
            "refactor": "renaming / restructuring",
            "revert": "revert previously commited code",
            "style": "white-space, formatting, semi-colons, etc.",
            "test": "any work to tests",
            "wip": "work in progress / might not build",
        }
        super().__init__(self.meta_dict.keys(), meta_dict=self.meta_dict, WORD=False)


class FooterCompleter(FuzzyWordCompleter):
    def __init__(self):
        self.github_meta_info = "Closes GitHub #issue"
        self.footer_meta_dict = {
            "Close #": self.github_meta_info,
            "Closes #": self.github_meta_info,
            "Closed #": self.github_meta_info,
            "Fix #": self.github_meta_info,
            "Fixes #": self.github_meta_info,
            "Fixed #": self.github_meta_info,
            "Resolve #": self.github_meta_info,
            "Resolves #": self.github_meta_info,
            "Resolved #": self.github_meta_info,
            "Clubhouse [ch": "Associates commit with ticket",
            "Clubhouse [branch ch": "Associates branch with ticket",
        }
        super().__init__(
            self.footer_meta_dict.keys(), meta_dict=self.footer_meta_dict, WORD=False
        )



#!/usr/bin/env python
#
# gitcommit: a tool for writing conventional commits, conveniently
# Author: Ben Greenberg
# Created: 20th September 2019
#
# Implements Conventional Commit v1.0.0-beta.4
#
# TEMPLATE:
#
# <type>[(optional scope)]: <description>
#
# [BREAKING CHANGE: ][optional body / required if breaking change]
#
# [optional footer]
#
# ADDITIONAL RULES:
# - Subject line (i.e. top) should be no more than 50 characters.
# - Every other line should be no more than 72 characters.
# - Wrapping is allowed in the body and footer, NOT in the subject.
#

from __future__ import print_function
import os
import sys
import subprocess
import textwrap
from prompt_toolkit import PromptSession, prompt, ANSI
from prompt_toolkit.history import FileHistory
from prompt_toolkit.application.current import get_app
from prompt_toolkit.styles import Style
from gitcommit.style import Ansi
from gitcommit.validators import (
    DescriptionValidator,
    TypeValidator,
    YesNoValidator,
    BodyValidator,
    FooterValidator,
)
from gitcommit.completers import TypeCompleter, FooterCompleter
from gitcommit.updater import check_for_update
from gitcommit.utils import capitaliseFirst
from gitcommit.style import style

home = os.path.expanduser("~")
CONFIG_HOME_DIR = os.path.join(home, ".gitcommit/")

IS_BREAKING_CHANGE = None  # default for global variable
try:
    WINDOW_WIDTH, _ = os.get_terminal_size()
except:
    WINDOW_WIDTH = 80  # default


class LineLengthPrompt:
    def __init__(self, length_limit, session):
        self.limit = length_limit
        self.session = session
        self.invalid_style = Style.from_dict({"rprompt": "bg:#ff0066 #000000"})
        self.valid_style = Style.from_dict({"rprompt": "bg:#b0f566 #000000"})

    def get_text(self):
        text = get_app().current_buffer.text
        cur_len = len(text)

        if cur_len > self.limit:
            self.session.style = self.invalid_style
            return f" {cur_len}/{self.limit} chars "

        else:
            self.session.style = self.valid_style
            return f" {cur_len}/{self.limit} chars "


def wrap_width(string):
    def format_string(string):
        string_lines = string.split("\n")
        string_lines_wrapped = []
        for line in string_lines:
            string_lines_wrapped += textwrap.wrap(
                line,
                width=WINDOW_WIDTH,
                break_long_words=False,
                replace_whitespace=False,
            )
        return "\n".join(string_lines_wrapped)

    if type(string) is list:
        return "\n".join([format_string(s) for s in string])
    elif type(string) is str:
        return format_string(string)
    else:
        raise TypeError("Unable to parse string argument")


def add_type(commit_msg):
    valid_types = {
        "feat": "MUST be used the commit adds/builds toward a new feature",
        "fix": "MUST be used when a commit represents a bug fix",
        "chore": "Any change to build system, dependencies, config files, scripts (no production code)",
        "docs": "Changes to documentation",
        "perf": "Changes that improves performance",
        "refactor": "Refactoring production code, e.g. renaming a variable/restructuring existing logic",
        "revert": "Any commit that explicitly reverts part/all changes introduced in a previous commit",
        "style": "Changes to white-space, formatting, missing semi-colons, etc.",
        "test": "Changes to tests e.g. adding a new/missing test or fixing/correcting existing tests",
        "wip": "Any code changes that are work in progress; they may not build (use these sparingly!)",
    }

    Ansi.print_info(
        wrap_width(
            [
                "Please specify the type of this commit using one of the available keywords. Accepted types: ",
                "TAB to autocomplete...",
            ]
        )
    )

    type_names = sorted(valid_types.keys())
    # create prefixes e.g. "    0  chore"
    prefixes = ["  " + str(i) + "  " + t for i, t in enumerate(type_names)]
    prefix_length = max([len(p) for p in prefixes]) + 2

    for i in range(len(type_names)):
        # type descriptions
        type_descr_width = WINDOW_WIDTH - prefix_length
        descr_lines = textwrap.wrap(
            valid_types[type_names[i]],
            width=type_descr_width,
            subsequent_indent=" " * prefix_length,
            break_long_words=False,
        )
        # ensure each line has trailing whitespace, then do join
        type_descr_str = "".join(map(lambda l: l.ljust(type_descr_width), descr_lines))

        # Combine type name with type description
        type_print = prefixes[i].ljust(prefix_length) + type_descr_str

        # Print the type
        Ansi.print_warning(type_print)

    valid_numeric_types = [str(n) for n in range(len(type_names))]
    valid_inputs = list(valid_types.keys()) + valid_numeric_types

    print()
    text = Ansi.b_green("Type: ")
    history_file_path = os.path.join(CONFIG_HOME_DIR, "type_history")
    c_type = prompt(
        ANSI(text),
        completer=TypeCompleter(),
        validator=TypeValidator(valid_inputs),
        history=FileHistory(history_file_path),
    )

    # Convert from number back to proper type name
    if c_type in valid_numeric_types:
        c_type = type_names[int(c_type)]

    commit_msg += c_type
    return commit_msg


def add_scope(commit_msg):
    Ansi.print_info(
        wrap_width(
            "\nWhat is the scope / a noun describing section of repo? (try to keep under 15 characters)"
        )
    )
    text = Ansi.colour(Ansi.fg.bright_green, "Scope (optional): ")
    history_file_path = os.path.join(CONFIG_HOME_DIR, "scope_history")
    c_scope = prompt(ANSI(text), history=FileHistory(history_file_path)).strip()

    if c_scope != "":
        commit_msg += "({})".format(c_scope)

    return commit_msg


def check_if_breaking_change():
    global IS_BREAKING_CHANGE  # required to be able to write to variable
    contains_break = ""
    print()  # breakline from previous section
    while True:
        text = Ansi.b_yellow("Does commit contain breaking change? (no) ")
        contains_break = (
            prompt(ANSI(text), validator=YesNoValidator(answer_required=False))
            .lower()
            .strip()
        )
        if contains_break == "":  # default
            IS_BREAKING_CHANGE = False
            break
        elif contains_break in ["y", "yes"]:
            IS_BREAKING_CHANGE = True
            return True
        else:
            IS_BREAKING_CHANGE = False
            return False


def add_description(commit_msg):
    if IS_BREAKING_CHANGE is None:
        raise ValueError("Global variable `IS_BREAKING_CHANGE` has not been set.")

    if IS_BREAKING_CHANGE:
        commit_msg += "!: "
    else:
        commit_msg += ": "

    num_chars_remaining = 50 - len(commit_msg)
    Ansi.print_info(
        wrap_width(
            "\nWhat is the commit description / title. A short summary of the code changes. Use the imperative mood. No more than {} characters.".format(
                num_chars_remaining
            )
        )
    )

    c_descr = ""

    history_file_path = os.path.join(CONFIG_HOME_DIR, "description_history")
    session = PromptSession(history=FileHistory(history_file_path))

    length_prompt = LineLengthPrompt(num_chars_remaining, session)
    while c_descr == "":
        text = Ansi.b_green("Description: ")
        c_descr = session.prompt(
            ANSI(text),
            validator=DescriptionValidator(num_chars_remaining),
            rprompt=length_prompt.get_text,
        )

    # Sanitise
    c_descr = c_descr.strip()  # remove whitespace
    c_descr = capitaliseFirst(c_descr)  # capital first letter
    if c_descr[-1] == ".":
        c_descr = c_descr[:-1]  # remove period if last character
        c_descr = c_descr.strip()  # remove further whitespace

    commit_msg += c_descr
    return commit_msg


def custom_prompt_continuation(width, line_number, is_soft_wrap):
    text_continuation = " " * (width - 2) + "┃ "
    return ANSI(Ansi.colour(Ansi.fg.bright_green, text_continuation))


def add_body(commit_msg):
    if IS_BREAKING_CHANGE is None:
        raise ValueError("Global variable `IS_BREAKING_CHANGE` has not been set.")

    history_file_path = os.path.join(CONFIG_HOME_DIR, "body_history")
    session = PromptSession(
        prompt_continuation=custom_prompt_continuation,
        history=FileHistory(history_file_path),
    )
    body_validator = BodyValidator(session, IS_BREAKING_CHANGE)

    if IS_BREAKING_CHANGE:
        Ansi.print_info(
            wrap_width(
                [
                    "\nExplain what has changed in this commit to cause breaking changes.",
                    "Press Esc before Enter to submit.",
                ]
            )
        )
        # text = Ansi.b_green("Body (required) ┃ ")
        text = Ansi.colour(Ansi.fg.bright_green, Ansi.bold, "Body ┃ ")
    else:
        Ansi.print_info(
            wrap_width(
                [
                    "\nProvide additional contextual information about the changes here.",
                    "Press Esc before Enter to submit.",
                ]
            )
        )
        text = Ansi.colour(Ansi.fg.bright_green, "Body (optional) ┃ ")

    c_body = session.prompt(ANSI(text), validator=body_validator)
    c_body = c_body.strip()  # remove leading/trailing whitespace
    c_body = capitaliseFirst(c_body)  # capital first letter

    if c_body != "":
        if IS_BREAKING_CHANGE:
            c_body = "BREAKING CHANGE: " + c_body

        b_lines = c_body.split("\n")
        num_blank_lines = 0  # track the number of consecutive blank lines

        condensed_b_lines = []
        for line in b_lines:
            l_stripped = line.strip()
            if l_stripped == "":
                num_blank_lines += 1
            else:
                num_blank_lines = 0

            if num_blank_lines > 1:
                continue  # ignore any blank lines after the first
            elif l_stripped == "":
                # skip any processing and add to output if blank
                condensed_b_lines.append(l_stripped)
            else:
                # check if we are dealing with a bulleted line
                bulleted_line = False
                if l_stripped[0] in ["*", "-"]:
                    bulleted_line = True
                    line_after_bullet = l_stripped[1:].strip()
                    l_stripped = " " + l_stripped[0] + " " + line_after_bullet

                # format each line with forced line breaks to maintain maximum line length
                wrapped_line = "\n".join(
                    textwrap.wrap(
                        l_stripped,
                        width=72,
                        break_long_words=False,
                        subsequent_indent="   " if bulleted_line else "",
                    )
                )
                condensed_b_lines.append(wrapped_line)

        # recombine all user defined lines
        full_body = "\n".join(condensed_b_lines)

        # append to commit message
        commit_msg += "\n\n" + full_body

    return commit_msg


def add_footer(commit_msg):
    Ansi.print_info(
        wrap_width(
            [
                "\nThe footer MUST contain meta-information about the commit:",
                " - Related pull-requests, reviewers, breaking changes",
                " - GitHub close/fix/resolve #issue or username/repository#issue",
                " - One piece of meta-information per-line",
                " - To submit, press the Esc key before Enter",
            ]
        )
    )

    text = Ansi.colour(Ansi.fg.bright_green, "Footer (optional) ┃ ")
    history_file_path = os.path.join(CONFIG_HOME_DIR, "footer_history")
    session = PromptSession(
        completer=FooterCompleter(),
        multiline=False,
        prompt_continuation=custom_prompt_continuation,
        history=FileHistory(history_file_path),
    )
    c_footer = session.prompt(ANSI(text), validator=FooterValidator(session)).strip()

    if c_footer != "":
        f_lines = c_footer.split("\n")
        f_lines = [
            l for l in f_lines if l.strip() != ""
        ]  # remove any lines that are empty: ""

        for i, line in enumerate(f_lines):
            line = line.strip()  # clean up extraneous whitespace

            # format each line with forced line breaks to maintain maximum line length
            f_lines[i] = "\n".join(
                textwrap.wrap(
                    line, width=72, break_long_words=False, subsequent_indent="  "
                )
            )

        # recombine all user defined footer lines
        formatted_footer = "\n".join(f_lines)
        commit_msg += "\n\n" + formatted_footer

    return commit_msg


def run():
    # print(sys.version + "/n")
    # Ensure the config directory exists
    os.makedirs(CONFIG_HOME_DIR, exist_ok=True)

    if WINDOW_WIDTH < 80:
        Ansi.print_error(
            f"It is recommended you increase your window width ({WINDOW_WIDTH}) to at least 80."
        )

    print("Starting a conventional git commit...")
    print(
        Ansi.colour(
            Ansi.fg.bright_red, "Tip: Press the up arrow key to recall history!"
        )
    )

    commit_msg = ""
    commit_msg = add_type(commit_msg)
    commit_msg = add_scope(commit_msg)
    check_if_breaking_change()
    commit_msg = add_description(commit_msg)
    commit_msg = add_body(commit_msg)
    commit_msg = add_footer(commit_msg)

    Ansi.print_ok("\nThis is your commit message:")
    print()
    print(commit_msg)
    print()

    # print("\nNOTE: This was a dry run and no commit was made.\n")

    # Extra command line arguments for git commit
    argv_passthrough = []  # by default add no extra arguments
    if len(sys.argv) > 1:
        argv_passthrough = sys.argv[1:]  # overwrite default list
    existing_args = " ".join(argv_passthrough)
    text = Ansi.colour(Ansi.fg.bright_yellow, "Extra args for git commit: ")
    extra_args_str = prompt(ANSI(text), default=existing_args)
    if extra_args_str != "":
        argv_passthrough = extra_args_str.split(" ")
    else:
        argv_passthrough = []

    # Ask for confirmation to commit
    confirmation_validator = YesNoValidator(answer_required=True)

    text = Ansi.b_yellow("Do you want to make your commit? [y/n] ")
    confirm = prompt(ANSI(text), validator=confirmation_validator).lower()

    if confirm in confirmation_validator.confirmations:
        print()
        cmds = ["git", "commit", "-m", commit_msg] + argv_passthrough
        returncode = subprocess.run(cmds).returncode
        if returncode == 0:
            Ansi.print_ok("\nCommit has been made to conventional commits standards!")
        else:
            Ansi.print_error("\nThere was an error whilst attempting the commit!")

    elif confirm in confirmation_validator.rejections:
        print("Aborting the commit...")

    check_for_update()


def main():
    try:
        run()
    except KeyboardInterrupt:
        print("\nAborted.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gitcommit']

package_data = \
{'': ['*']}

install_requires = \
['packaging>=19.2,<20.0',
 'prompt_toolkit>=3.0,<4.0',
 'pyperclip>=1.7,<2.0',
 'requests>=2.22,<3.0']

entry_points = \
{'console_scripts': ['gitcommit = gitcommit.gitcommit:main']}

setup_kwargs = {
    'name': 'conventional-commit',
    'version': '0.3.3',
    'description': 'a tool for writing conventional commits, conveniently',
    'long_description': '<p  align="center">\n  <strong>gitcommit</strong>\n  <br>\n  <code>a tool for writing conventional commits, conveniently</code>\n  <br><br>\n  <a href="https://badge.fury.io/py/conventional-commit"><img src="https://badge.fury.io/py/conventional-commit.svg" alt="PyPI version" height="18"></a>\n  <a href="https://travis-ci.org/nebbles/gitcommit/branches"><img src="https://travis-ci.org/nebbles/gitcommit.svg?branch=master" alt="Travis CI build" height="18"></a>\n  <a href="https://github.com/psf/black"><img src="https://img.shields.io/badge/code%20style-black-000000.svg" alt="Code style: black" height="18"></a>\n</p>\n\n# Install\n\nTo install\n\n```\npip install conventional-commit\n```\n\nTo use, run the following command from within a git repository\n\n```\ngitcommit\n```\n\n# Overview\n\nThe purpose of this utility is to expedite the process of committing with a conventional message format in a user friendly way. This tool is not templated, because it sticks rigidly to the [Conventional Commit standard](https://www.conventionalcommits.org), and thus not designed to be \'altered\' on a case by case basis.\n\nCommit messages produced follow the general template:\n\n```\n<type>[(optional scope)]: <description>\n\n[BREAKING CHANGE: ][optional body / required if breaking change]\n\n[optional footer]\n```\n\nAdditional rules implemeted:\n\n1. Subject line (i.e. top) should be no more than 50 characters.\n2. Every other line should be no more than 72 characters.\n3. Wrapping is allowed in the body and footer, NOT in the subject.\n\n# Development\n\nThe old distribution method is documented in\n[docs/dev_distibution_legacy.md](docs/dev_distribution_legacy.md)\n\n_Note: if modifying `.travis.yml` you should verify it by running `travis lint .travis.yml`_\n\n## Getting started\n\n1. Make sure you have [pre-commit](https://pre-commit.com/#install) installed.\n\n1. Make sure you have [pyenv](https://github.com/pyenv/pyenv) installed.\n\n1. Make sure you have [Poetry](https://github.com/sdispater/poetry) installed.\n\n1. `git clone`\n\n1. `pre-commit install`\n\n1. It is highly recommend you enable setting for storing the venvs within your projects.\n\n   ```\n   poetry config virtualenvs.in-project true\n   ```\n\n1. Install project dependencies.\n   ```\n   poetry install\n   ```\n\n## Running the package locally\n\n1. Activate the virtual environment.\n\n   ```\n   source .venv/bin/activate\n   ```\n\n1. Run the package as a module.\n   ```\n   python -m gitcommit\n   ```\n\nAlternatively,\n\n1. Run the package using Poetry\'s venv as context\n   ```\n   poetry run python -m gitcommit\n   ```\n\nOr, if in another directory,\n\n1.  ```\n    ~/GitHub/gitcommit/.venv/bin/python -m gitcommit\n    ```\n\n## Deploy\n\nDeployment is handled automatically by Travis CI. It has been linked to the\nrepository and is automatically watching for pushes to master. It will build\nand test every commit to master. It will also build every tagged commit as\nif it was a branch, and since its a tagged commit, will attempt to publish\nit to PyPI.\n\n1. Don\'t forget to increment version number set in `pyproject.toml`. This can be\n   done with poetry.\n\n   ```\n   poetry version [patch|minor|major]\n   ```\n\n1. Update the version number as stored in the `gitcommit/__version__.py` file to match that designated by Poetry.\n\n1. Tag the commit (by default applies to HEAD commit - make sure you are on the latest `develop` commit).\n\n   ```\n   git tag v#.#.#\n   ```\n\n1. When pushing commits to remote, you must explicitly push tags too.\n   ```\n   git push origin --tags\n   ```\n\n## Acknowledgements\n\nThis work takes inspiration from [another repository porting Commitizen to Python](https://github.com/Woile/commitizen). This repository however uses none of the same source code and is focusing on a different approach.\n\n## License\n\nThis work is published under [GNU GPLv3](./LICENSE).\n',
    'author': 'nebbles',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nebbles/gitcommit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



def capitaliseFirst(string_in: str) -> str:
    """A utility function that will capitalise only the first character in a string. Unlike str.capitalize() which will lower all other characters in the string."""
    if len(string_in) == 0:
        return ""
    elif len(string_in) == 1:
        return string_in[0].upper()
    elif len(string_in) > 1:
        return string_in[0].upper() + string_in[1:]
